#!/usr/local/bin/gosh

;; Taking a single argument isbn, the title of whose book will be saved.

(use daemon)
(use gauche.logger)
(use gauche.net)
(use gauche.selector)
(use rfc.http)
(use srfi-1)
(use srfi-13)
(use sxml.ssax)
(use sxml.sxpath)
(use aws)

(define *port* 3203)
(define *max-length* 13)

(define *namespace*
  '((aws . "http://webservices.amazon.com/AWSECommerceService/2005-10-05")))

(define (run)
  (let ((selector (make <selector>))
        (server (make-server-socket 'inet *port* :reuse-addr? #t)))
    (define (accept-handler sock flag)
      (let* ((client (socket-accept server))
             (sockaddr (socket-address client))
             (name (sockaddr-name sockaddr)))
        (if (string-prefix? "127.0.0.1:" name)
            (let ((output (socket-output-port client)))
              (selector-add! selector
                             (socket-input-port client :buffering #f)
                             (lambda (input flag)
                               (isbn->term client input output))
                             '(r)))
            (socket-shutdown client 2))))
    (define (isbn->term client input output)
      (let ((isbn (read-block *max-length* input)))
        (cond ((eof-object? isbn)
               (selector-delete! selector input #f #f)
               (socket-close client))
              (else
               (let ((query (aws-item-lookup :item-id isbn :id-type "ISBN" :search-index "Books")))
                 (cond ((string=? "200" (execute query))
                        (call-with-input-string (ref query 'body)
                          (lambda (iport)
                            (let ((sxml (ssax:xml->sxml iport *namespace*))
                                  (q-title (sxpath '(// aws:Title))))
                              (or (and-let* ((tree (q-title sxml))
                                             ((= 1 (length tree))))
                                    (format output "title('~a').~%"
                                            (regexp-replace-all* (second (car tree))
                                                                 #/\\/ "\\\\\\"
                                                                 #/'/ "\\\\'"))
                                    (flush output)
                                    'success)
                                  (begin
                                    (format output "false.~%")
                                    (flush output)
                                    'failed))))))
                       (else
                        (format output "~d.~%" (ref query 'status))
                        (flush output))))))))

    (selector-add! selector
                   (socket-fd server)
                   accept-handler
                   '(r))
    (do () (#f) (selector-select selector))))

(define (main args)
  (log-open "/home/tabe/work/typo/log/isbin-to-title.log")
  (aws-locale 'jp)
  (aws-access-key-id "********************")
  (daemon-pid-file "/home/tabe/work/typo/log/isbn-to-title.pid")
  (daemonize)
  (run)
  0)
