#!/usr/bin/env gosh

(use daemon)
(use gauche.listener)
(use gauche.logger)
(use gauche.net)
(use gauche.selector)
(use sendmail)
(use srfi-13)

(define *port* 7654)
(define *max-length* 8192)

(define (dispatch)
  (sys-umask #o022)
  (sendmail-sender "tabe@fixedpoint.jp")
  (let ((selector (make <selector>))
        (server (make-server-socket 'inet *port* :reuse-addr? #t)))
    (define (accept-handler sock flag)
      (let* ((client (socket-accept server))
             (sockaddr (socket-address client))
             (name (sockaddr-name sockaddr)))
        (if (string-prefix? "127.0.0.1:" name)
            (let ((output (socket-output-port client))
                  (input (socket-input-port client :buffering #f)))
              (selector-add! selector
                             input
                             (lambda (input flag)
                               (call client input output))
                             '(r))
              (log-format "selector-add!")
              )
            (socket-shutdown client 2))))
    (define (call client input output)
      (define (close)
        (selector-delete! selector input #f #f)
        (socket-close client))
      (define (fail)
        (display "false.\n" output)
        (flush output)
        (close))
      (define (success)
        (display "true.\n" output)
        (flush output)
        (close))
      (let ((q (string-incomplete->complete (read-block *max-length* input))))
        (cond ((or (not q)
                   (eof-object? q))
               (close))
              ((complete-sexp? q)
               (selector-delete! selector input #f #f)
               (let ((query (read-from-string q)))
                 (case (car query)
                   ((sendmail)
                    (log-format "~s" query)
                    (guard (e
                            (else
                             (fail)
                             (log-format "~s" e)
                             ))
                      (eval query (interaction-environment))
                      (success))
                    ))))
              (else
               (close)))))
    (selector-add! selector
                   (socket-fd server)
                   accept-handler
                   '(r))
    (do () (#f) (selector-select selector))))

(define (main args)
  (log-open "/home/tabe/work/typo/log/pigeon.log")
  (daemon-pid-file "/home/tabe/work/typo/log/pigeon.pid")
  (daemonize)
  (dispatch)
  0)
