#!/usr/bin/env gosh
;; -*- mode: scheme -*-

(use dbi)
(use file.util)
(use gauche.parameter)
(use gauche.parseopt)
(use lang.swi-prolog)
(use srfi-1)
(use srfi-13)
(use text.inflection)
(use text.tree)
(use typo.dbi)
(use util.list)
(use util.relation)

(define typo-root-directory (make-parameter "/home/tabe/work/typo"))

(define conn (make-parameter #f))
(define tables (make-parameter #f))
(define leading-columns (make-parameter #f))
(define optional-columns (make-parameter #f))

(define (dump-table name)
  (let* ((result (typo-dbi-select (conn) "information_schema.columns"
                                  '(:column_name :ordinal_position :is_nullable)
                                  :table_schema "public" :table_name name))
         (accessor (relation-accessor result)))
    (unwind-protect
     (let lp ((rest (sort (relation-rows result)
                          (lambda (c1 c2) (string<? (accessor c1 "ordinal_position") (accessor c2 "ordinal_position")))))
              (columns '()))
       (cond ((null? rest)
              (leading-columns (reverse columns))
              (optional-columns '()))
             ((car rest)
              => (lambda (c)
                   (let ((column-name (accessor c "column_name")))
                     (cond ((and (or (string=? "id" column-name)
                                     (string-suffix? "_id" column-name))
                                 (string=? "NO" (accessor c "is_nullable")))
                            (lp (cdr rest) (cons column-name columns)))
                           (else
                            (leading-columns (reverse columns))
                            (optional-columns (map (cut accessor <> "column_name") rest)))))))
             (else
              (error "can not happen" name))))
     (dbi-close result)))
  (let* ((result (typo-dbi-select (conn) name '()))
         (accessor (relation-accessor result)))
    (define (field-value row c)
      (let ((v (accessor row c)))
        (if (string-null? v)
            #f
            v)))
    (define (dump-row oport row)
      (let ((ids (map (lambda (c) (string->number (accessor row c))) (leading-columns)))
            (alist (map (lambda (c) (cons c (field-value row c))) (optional-columns))))
        (write
         (append ids (append-map (lambda (pair) (list (make-keyword (car pair)) (cdr pair))) alist))
         oport)
        (newline oport)))
    (call-with-output-file (build-path (typo-root-directory) "db" "dump" (format #f "~a.scm" name))
      (lambda (oport)
        (for-each
         (cut dump-row oport <>)
         (relation-rows result))))))

(define (main args)
  (conn (dbi-connect "dbi:pg:dbname=typo;host=localhost;port=5432" :username "tabe"))
  (tables (let* ((result (typo-dbi-select (conn) "information_schema.tables" '(:table_name) :table_schema "public" :table_type "BASE TABLE"))
                 (accessor (relation-accessor result)))
            (begin0
              (map (cut accessor <> "table_name") (relation-rows result))
              (dbi-close result))))
  (for-each dump-table (tables))
  (dbi-close (conn #f))
  0)
