(define-module aws.error
  (use gauche.sequence)
  (export-all)
  )

(select-module aws.error)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Programming Guide > Finding Items to Buy > Handling Errors and Troubleshooting Applications
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Error Codes and Messages

(define-condition-type <aws-error> <error>
  aws-error?
  (code code))
(define-condition-type <aws-ecs-error> <aws-error>
  aws-ecs-error?
  )

;; A2S returns error messages in 
;; - English for US, UK, DE, FR, and CA locales
;; - Japanese for the Amazon.co.jp (JP) locale

(define-macro (define-aws-error . identifies)
  `(begin
     ,@(map-with-index
        (lambda (i id)
          `(define ,(string->symbol (format #f "*aws-error-~a*" id)) ,i))
        identifies)))

(define-aws-error
  exact-parameter-requirement
  exceeded-maximum-parameter-values
  insufficient-parameter-values
  internal-error
  invalid-enumrated-parameter
  invalid-iso8601-time
  invalid-operation-for-marketplace
  invalid-operation-parameter
  invalid-parameter-combination
  invalid-parameter-value
  invalid-response-group
  invalid-service-parameter
  invalid-subscription-id
  invalid-xslt-address
  maximum-parameter-requirement
  minimum-parameter-requirement
  missing-operation-parameter
  missing-parameter-combination
  missing-parameters
  missing-parameter-value-combination
  missing-service-parameter
  parameter-out-of-range
  parameter-repeated-in-request
  restricted-parameter-value-combination
  xslt-transformation-error
  )

(define-macro (define-aws-ecs-error . identifies)
  `(begin
     ,@(map-with-index
        (lambda (i id)
          `(define ,(string->symbol (format #f "*aws-ecs-error-~a*" id)) ,i))
        identifies)))

(define-aws-ecs-error
  exceeded-maximum-cart-items
  invalid-cart-id
  invalid-hmac
  invalid-quality
  item-already-in-cart
  item-not-accessible
  item-not-eligible-for-cart
  no-exact-matches
  no-similarities
  )

(provide "aws/error")
