(define-module typo.html
  (use srfi-1)
  (use srfi-19) ; time and date
  (use text.html-lite)
  (use text.tree)
  (use util.list)
  (use www.cgi)
  (use www.cgi.session.file)
  (use www.cgi.misc)
  (export
   year-select
   month-select
   day-select
   hour-select
   minute-select
   ymd-select
   ymdhm-select

   typo-css
   typo-js
   typo-title
   typo-head
   typo-body
   typo-main
   typo-error-handler
   ))

(select-module typo.html)

(define (year-select name . opt)
  (let-keywords* opt ((date (current-date))
                      (selected (date-year date)))
    (let ((y (date-year date)))
      (html:select
       :name name
       (map (lambda (option)
              (html:option :value option :selected (and (= selected option) selected) (format #f "~2'0d" option)))
            (iota 5 (- y 2)))))))

(define-macro (make-time-selector %field% %length% %base%)
  (let ((%proc% (string->symbol (format #f "date-~a" %field%))))
    `(lambda (name . opt)
       (let-keywords* opt ((date (current-date))
                           (selected (,%proc% date)))
         (html:select
          :name name
          (map (lambda (option)
                 (html:option :value option :selected (and (= selected option) selected) (format #f "~2'0d" option)))
               (iota ,%length% ,%base%)))))))

(define month-select  (make-time-selector month  12 1))
(define day-select    (make-time-selector day    31 1))
(define hour-select   (make-time-selector hour   24 0))
(define minute-select (make-time-selector minute 60 0))

(define (ymd-select year month day)
  (let ((date (current-date)))
    (list
     (year-select year :date date)
     "-"
     (month-select month :date date)
     "-"
     (day-select day :date date))))

(define (ymdhm-select year month day hour minute)
  (let ((date (current-date)))
    (list
     (year-select year :date date)
     "-"
     (month-select month :date date)
     "-"
     (day-select day :date date)
     " "
     (hour-select hour :date date)
     ":"
     (minute-select minute :date date))))

(define (typo-css)
  (map (cut html:link :rel "Stylesheet" :href <> :type "text/css")
       '("/public/typo.css")))

(define (typo-js)
  (map (cut html:script :src <> :type "text/javascript")
       '("/public/prototype.js")))

(define (typo-title conn params)
  "typo.fixedpoint.jp")

(define (typo-head conn params)
  (html:head
   (typo-css)
   (typo-js)
   (html:title (typo-title conn params))))

(define (typo-body conn params)
  (html:body
   (html:h1 (typo-title conn params))
   (html:p "Hello, Typo.")))

(define (%main conn params session action)
  `(,(cgi-header)
    ,(html-doctype)
    ,(html:html
      (typo-head conn params)
      (typo-body conn params))))

(define (typo-main conn params)
  (*session-directory* "/home/tabe/work/typo/tmp") ; FIXME
  (let ((session (session-begin <cgi-session-file>)))
    (and-let* ((id (cgi-get-parameter "id" params)))
      (or (and-let* ((val (session-get session 'id)))
            (session-set session 'id (cons id val)))
          (session-set session 'id (list id))))
    (let ((url (or
                (cgi-get-parameter "url" params) ; FIXME
                (cgi-get-metavariable "PATH_INFO")))
          (rx #/^\/?([a-zA-Z0-9\-\/]*)$/))
      (rxmatch-case url
        (rx
         (#f action)
         (%main conn params session action))
        (else
         (cgi-header :location "/"))))))

(define (typo-error-handler e . rest)
  (let ((m (condition-ref e 'message)))
    `(,(cgi-header)
      ,(html-doctype)
      ,(html:html
        (html:head
         (html:title m))
        (html:body
         (html:p m))))))

(provide "typo/html")
