#!/usr/bin/env gosh

(use aws)
(use rfc.http)
(use srfi-13)
(use sxml.ssax)
(use sxml.sxpath)
(use typo.script)

(define *search-index-support-by-locale*
  (aws-developer-guide "/APPNDX_SearchIndexValues.html"))

(define *namespace*
  '((fo . "http://www.w3.org/1999/XSL/Format")
    (xhtml . "http://www.w3.org/1999/xhtml")))

(define (main args)
  (typo-script-aws-keyword-file (cadr args))
  (typo-script-aws-keyword-on)
  (receive (code headers body)
      (http-get *aws-documentation-host-name* *search-index-support-by-locale*)
    (cond ((string-prefix? "2" code)
           (call-with-input-string body
             (lambda (iport)
               (let ((sxml (ssax:xml->sxml iport *namespace*))
                     (q-table (sxpath '(// (xhtml:div (@ (equal? (class "informaltable")))) xhtml:table)))
                     (q-locales (sxpath '(// xhtml:thead xhtml:tr xhtml:th *text*)))
                     (q-indices (sxpath '(// xhtml:tbody xhtml:tr)))
                     )
                 (let* ((table (q-table sxml))
                        (locales (map (compose string->symbol string-downcase) (cdr (q-locales table))))
                        (indices (q-indices table))
                        )
                   (print ";; locales --------------------------------------------------------------")
                   (format (current-output-port) "~s~%" locales)
                   (print ";; indices ------------------------------------------------------------------------")
                   (for-each
                    (lambda (row)
                      (let* ((row ((sxpath '(// xhtml:td)) row))
                             (index (string->symbol (typo-script-aws-dasherize (cadar row))))
                             (q-support? (sxpath '(// xhtml:div))))
                        (format (current-output-port) "~s~%" (cons index (map (lambda (td) (not (null? (q-support? td)))) (cdr row))))))
                    indices)
                   ))))
           )
          (else
           (error "http failure:" code)
           )))
  (typo-script-aws-keyword-off)
  0)
