#!/usr/bin/env gosh

(use aws)
(use rfc.http)
(use srfi-13)
(use sxml.ssax)
(use sxml.sxpath)
(use typo.script)

(define (search-index-param locale)
  (aws-developer-guide (format #f "/~aSearchIndexParamForItemsearch.html" (string-upcase (x->string locale)))))

(define *namespace*
  '((fo . "http://www.w3.org/1999/XSL/Format")
    (xhtml . "http://www.w3.org/1999/xhtml")))

(define (main args)
  (typo-script-aws-keyword-file (cadr args))
  (typo-script-aws-keyword-on)
  (for-each
   (lambda (locale)
     (receive (code headers body)
         (http-get *aws-documentation-host-name* (search-index-param locale))
       (cond ((string-prefix? "2" code)
              (format (current-output-port) "(~a ; ----------------------------------------------------------------------~%" locale)
              (call-with-input-string body
                (lambda (iport)
                  (let ((sxml (ssax:xml->sxml iport *namespace*))
                        (q-h2 (sxpath '(// xhtml:h2 xhtml:a @ id)))
                        (q-tbody (sxpath '(// (xhtml:div (@ (equal? (class "informaltable")))) xhtml:table xhtml:tbody)))
                        (q-item (sxpath '(// xhtml:ul xhtml:li xhtml:p *text*)))
                        )
                    (for-each
                     (lambda (h2 tbody)
                       (format (current-output-port) " (~a~%" (typo-script-aws-dasherize (cadr (string-split (cadr h2) #\_))))
                       (for-each
                        (lambda (index) (format (current-output-port) "  ~a~%" (typo-script-aws-dasherize index)))
                        (q-item tbody))
                       (print "  )")
                       )
                     (cdr (q-h2 sxml))
                     (q-tbody sxml)
                     ))))
              (print ")")
              )
             (else
              (error "http failure:" code)
              ))))
   *aws-locales*)
  (typo-script-aws-keyword-off)
  0)
