:- module(lcs,
          [ lcs_with_positions/3,
            lcs/3,
            lcs_fold/7,
            lcs_edit_list/3
          ]).
:- encoding(utf8).

split_at(List,0,[],List) :- !.
split_at([Head|Tail],I,[Head|X],Y) :- I>0,!,
    I1 is I-1,
    split_at(Tail,I1,X,Y).

v(V,M_N,K,Vm_n_k) :-
    M_N_K is M_N+K,
    nth0(M_N_K,V,Vm_n_k).
v(V,M_N,K,Vm_n_k,Vnew) :-
    M_N_K is M_N+K,
    split_at(V,M_N_K,Head,[_|Tail]),
    append(Head,[Vm_n_k|Tail],Vnew).
vlt(V,M_N,K) :-
    X is K-1,
    Y is K+1,
    v(V,M_N,X,Vx),
    v(V,M_N,Y,Vy),
    Vx < Vy.
finish(M_N,_,[I,Max,R],[Max,RR]) :- I>M_N,!,reverse(R,RR).
finish(M_N,[V_r,V_l],[I,Max,_],LCS) :-
    v(V_l,M_N,I,Zl),
    Zl>Max,!,
    I1 is I+1,
    v(V_r,M_N,I,Zr),
    finish(M_N,[V_r,V_l],[I1,Zl,Zr],LCS).
finish(M_N,[V_r,V_l],[I,Max,R],LCS) :-
    I1 is I+1,
    finish(M_N,[V_r,V_l],[I1,Max,R],LCS).
d_loop(_,_,_,_,M_N,_,D,_) :- D>M_N,!,
    throw(error(lcs_with_positions)).
d_loop(A,B,N,M,M_N,VList,D,LCS) :-
    K is -D,
    k_loop(A,B,N,M,M_N,VList,D,K,LCS).
k_loop(A,B,N,M,M_N,VList,D,K,LCS) :- K>D,!,
    D1 is D+1,
    d_loop(A,B,N,M,M_N,VList,D1,LCS).
k_loop(A,B,N,M,M_N,[V_d,V_r,V_l],D,K,LCS) :-
    ( (K =:= -D;K =\= D,vlt(V_d,M_N,K)) ->
      K1 is K+1,
      v(V_d,M_N,K1,X1),
      v(V_l,M_N,K1,L1),
      v(V_r,M_N,K1,R1)
    ; K1 is K-1,
      v(V_d,M_N,K1,X0),
      v(V_l,M_N,K1,L1),
      v(V_r,M_N,K1,R1),
      X1 is X0+1
    ),
    Y1 is X1-K,
    xy_loop([A,B,N,M],[X1,Y1,L1,R1],[X2,Y2,L2,R2]),
    v(V_d,M_N,K,X2,V_dx),
    v(V_r,M_N,K,R2,V_rx),
    v(V_l,M_N,K,L2,V_lx),
    ( X2>=N,Y2>=M ->
      I is -M_N,
      finish(M_N,[V_rx,V_lx],[I,0,[]],LCS)
    ; K2 is K+2,
      k_loop(A,B,N,M,M_N,[V_dx,V_rx,V_lx],D,K2,LCS)
    ).
xy_loop([A,B,N,M],[X,Y,L,R],XYLR) :- X<N,Y<M,nth0(X,A,Ax),nth0(Y,B,By),Ax = By,!,
    X1 is X+1,Y1 is Y+1,L1 is L+1,
    xy_loop([A,B,N,M],[X1,Y1,L1,[[Ax,X,Y]|R]],XYLR).
xy_loop(_,[X,Y,L,R],[X,Y,L,R]).

lcs_with_positions([],[],[0,[]]).
lcs_with_positions(A,B,LCS) :-
    length(A,N),
    length(B,M),
    M_N is N+M,
    L is (M_N)*2+1,
    findall(0,between(1,L,_),V_d),
    findall([],between(1,L,_),V_r),
    findall(0,between(1,L,_),V_l),
    d_loop(A,B,N,M,M_N,[V_d,V_r,V_l],0,LCS).

lcs(A,B,LCS) :- lcs_with_positions(A,B,[_,X]),maplist(nth0(0),X,LCS).

fold(_,Seed,[],Seed) :- !.
fold(Predicate,Seed,[X|Rest],Result) :-
    call(Predicate,X,Seed,Next),
    fold(Predicate,Next,Rest,Result).

lcs_fold(AOnly,BOnly,Both,Seed,A,B,Result) :-
    lcs_with_positions(A,B,[_,Common]),
    lcs_fold(AOnly,BOnly,Both,Common,Seed,A,0,B,0,Result).
lcs_fold(AOnly,BOnly,_,[],Seed,A,_,B,_,Result) :- !,
    fold(AOnly,Seed,A,X),
    fold(BOnly,X,B,Result).
lcs_fold(AOnly,BOnly,Both,[[Pos,AOff,BOff]|Rest],Seed,A,APos,B,BPos,Result) :-
    ASkip is AOff-APos,
    BSkip is BOff-BPos,
    AOff1 is AOff+1,
    BOff1 is BOff+1,
    split_at(A,ASkip,AHead,[_|CdrATail]),
    split_at(B,BSkip,BHead,[_|CdrBTail]),
    fold(AOnly,Seed,AHead,X),
    fold(BOnly,X,BHead,Y),
    call(Both,Pos,Y,Z),
    lcs_fold(AOnly,BOnly,Both,Rest,Z,CdrATail,AOff1,CdrBTail,BOff1,Result).

%% The following sub-predicate is expected to be local; a_only, b_only, both, and push_hunk.
%% They use attributed variables associated with a virtual module 'lcs_edit_list'.

a_only(APos,Elt,Hunk,[[-,N1,Elt]|Hunk]) :-
    get_attr(APos,lcs_edit_list,N),
    N1 is N+1,
    put_attr(APos,lcs_edit_list,N1).

b_only(BPos,Elt,Hunk,[[+,N1,Elt]|Hunk]) :-
    get_attr(BPos,lcs_edit_list,N),
    N1 is N+1,
    put_attr(BPos,lcs_edit_list,N1).

both(APos,BPos,Hunks,_,Hunk,[]) :-
    get_attr(APos,lcs_edit_list,M),
    M1 is M+1,
    put_attr(APos,lcs_edit_list,M1),
    get_attr(BPos,lcs_edit_list,N),
    N1 is N+1,
    put_attr(BPos,lcs_edit_list,N1),
    push_hunk(Hunks,Hunk).

push_hunk(_,[]) :- !.
push_hunk(Hunks,Hunk) :-
    get_attr(Hunks,lcs_edit_list,X),
    reverse(Hunk,XHunk),
    put_attr(Hunks,lcs_edit_list,[XHunk|X]).

lcs_edit_list(A,B,Result) :-
    put_attr(APos,lcs_edit_list,-1),
    put_attr(BPos,lcs_edit_list,-1),
    put_attr(Hunks,lcs_edit_list,[]),
    lcs_fold(a_only(APos),b_only(BPos),both(APos,BPos,Hunks),[],A,B,Last),
    push_hunk(Hunks,Last),
    get_attr(Hunks,lcs_edit_list,XHunks),
    reverse(XHunks,Result).
