%% -*- coding: utf-8 -*-
:- encoding(utf8).
:- begin_tests(sexp).
:- use_module(sexp).

test(lexeme,[setup(atom_chars(',@',X))]) :- phrase(sexp:lexeme(X),X).
test(lexeme,[setup(atom_chars(',',X))]) :- phrase(sexp:lexeme(X),X).

test(whitespace,[setup(atom_chars('\v',X))]) :- phrase(sexp:whitespace(X),X).

test(interlexeme_space,[setup(atom_chars('\v',X))]) :- phrase(sexp:interlexeme_space(X),X).
test(interlexeme_space,[setup(atom_chars(' \t ',X))]) :- phrase(sexp:interlexeme_space(X),X).

test(comment,[setup(atom_chars('; c\n',X))]) :- phrase(sexp:comment(X),X).
test(comment,[setup(atom_chars('; c\r\n',X))]) :- phrase(sexp:comment(X),X).
test(comment,[setup(atom_chars('#|comment|#',X))]) :- phrase(sexp:comment(X),X).
test(comment,[setup(atom_chars('#;(C)',X))]) :- phrase(sexp:comment(X),X).
test(comment,[setup(atom_chars('#;(+ 1 2)',X))]) :- phrase(sexp:comment(X),X).
test(comment,[setup(atom_chars('#!r6rs',X))]) :- phrase(sexp:comment(X),X).

test(proper_sign) :- phrase(sexp:proper_sign(['+']),['+']).
test(proper_sign) :- phrase(sexp:proper_sign(['-']),['-']).

test(sign) :- phrase(sexp:sign([]),[]).
test(sign) :- phrase(sexp:sign(['+']),['+']).
test(sign) :- phrase(sexp:sign(['-']),['-']).

test(digit) :- phrase(sexp:digit(['1']),['1']).

test(uinteger) :- phrase(sexp:uinteger(10,['1']),['1']).
test(uinteger,[setup(atom_chars('1023',X))]) :- phrase(sexp:uinteger(10,X),X).
test(ureal,[setup(atom_chars('19',X))]) :- phrase(sexp:ureal(10,X),X).
test(ureal,[setup(atom_chars('10/23',X))]) :- phrase(sexp:ureal(10,X),X).
test(ureal,[setup(atom_chars('.999',X))]) :- phrase(sexp:ureal(10,X),X).
test(ureal,[setup(atom_chars('3.141592',X))]) :- phrase(sexp:ureal(10,X),X).
test(ureal,[setup(atom_chars('3.141592|7',X))]) :- phrase(sexp:ureal(10,X),X).

test(real,[setup(atom_chars('1',X))]) :- phrase(sexp:real(10,X),X).

test(letter,[setup(atom_chars('A',X))]) :- phrase(sexp:letter(X),X).
test(letter,[setup(atom_chars('z',X))]) :- phrase(sexp:letter(X),X).

test(identifier,[setup(atom_chars('lambda',X))]) :- phrase(sexp:identifier(X),X).
test(identifier,[setup(atom_chars('q',X))]) :- phrase(sexp:identifier(X),X).
test(identifier,[setup(atom_chars('soup',X))]) :- phrase(sexp:identifier(X),X).
test(identifier,[setup(atom_chars('list->vector',X))]) :- phrase(sexp:identifier(X),X).
test(identifier,[setup(atom_chars('+',X))]) :- phrase(sexp:identifier(X),X).
test(identifier,[setup(atom_chars('V17a',X))]) :- phrase(sexp:identifier(X),X).
test(identifier,[setup(atom_chars('<=',X))]) :- phrase(sexp:identifier(X),X).
test(identifier,[setup(atom_chars('a34kTMNs',X))]) :- phrase(sexp:identifier(X),X).
test(identifier,[setup(atom_chars('->-',X))]) :- phrase(sexp:identifier(X),X).
test(identifier,[setup(atom_chars('the-word-recursion-has-many-meanings',X))]) :- phrase(sexp:identifier(X),X).
test(identifier,[setup(atom_chars('あいうえお',X))]) :- phrase(sexp:identifier(X),X).

test(boolean,[setup(atom_chars('#t',X))]) :- phrase(sexp:boolean(X),X).
test(boolean,[setup(atom_chars('#T',X))]) :- phrase(sexp:boolean(X),X).
test(boolean,[setup(atom_chars('#f',X))]) :- phrase(sexp:boolean(X),X).
test(boolean,[setup(atom_chars('#F',X))]) :- phrase(sexp:boolean(X),X).

test(character,[setup(atom_chars('#\\a',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\A',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\(',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\ ',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\nul',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\alarm',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\backspace',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\tab',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\linefeed',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\newline',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\vtab',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\page',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\return',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\esc',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\space',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\delete',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\xFF',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\x03BB',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\x00006587',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\λ',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\x0001z',X)),fail]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\λx',X)),fail]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\alarmx',X)),fail]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\Alarm',X)),fail]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\alert',X)),fail]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\xA',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\xff',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\(x)',X)),fail]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\(x',X)),fail]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\x00110000',X)),fail,blocked('bother to check range')]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\x000000001',X))]) :- phrase(sexp:character(X),X).
test(character,[setup(atom_chars('#\\xD800',X))],fail,blocked('bother to check range')) :- phrase(sexp:character(X),X).

test(string_element,[setup(atom_chars('\\  \n',X))]) :- phrase(sexp:string_element(X),X).
test(string_element,[setup(atom_chars('\\  \n \t ',X))]) :- phrase(sexp:string_element(X),X).

test(string,[setup(atom_chars('"abc"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\x41;bc"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\x41; bc"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\x41bc;"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\x41"',X)),fail]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\x;"',X)),fail,blocked('bother to check range')]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\x41bx;"',X)),fail,blocked('bother to check range')]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\x00000041;"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\x0010FFFF;"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\x00110000;"',X)),fail,blocked('bother to check range')]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\x000000001;"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\xD800;"',X)),fail,blocked('bother to check range')]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"A\nbc"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"あいうえお"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"あい\\"えお"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"あい\\nえお"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"あい\\nえお"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\  \n"',X))]) :- phrase(sexp:string(X),X).
test(string,[setup(atom_chars('"\\  \n \t "',X))]) :- phrase(sexp:string(X),X).

test(number,[setup(atom_chars('1',X))]) :- phrase(sexp:number(X),X).
test(number,[setup(atom_chars('3.1415926535898F0',X))]) :- phrase(sexp:number(X),X).
test(number,[setup(atom_chars('0.6L0',X))]) :- phrase(sexp:number(X),X).
test(number,[setup(atom_chars('1.1|53',X))]) :- phrase(sexp:number(X),X).
test(number,[setup(atom_chars('+inf.0',X))]) :- phrase(sexp:number(X),X).
test(number,[setup(atom_chars('-inf.0',X))]) :- phrase(sexp:number(X),X).
test(number,[setup(atom_chars('+nan.0',X))]) :- phrase(sexp:number(X),X).
test(number,[setup(atom_chars('-nan.0',X))]) :- phrase(sexp:number(X),X). % errata

test(list,[setup(atom_chars('(a b c d e)',X))]) :- phrase(sexp:list(X),X).
test(list,[setup(atom_chars('(a . (b . (c . (d . (e . ())))))',X))]) :- phrase(sexp:list(X),X).
test(list,[setup(atom_chars('()',X))]) :- phrase(sexp:list(X),X).
test(list,[setup(atom_chars('[]',X))]) :- phrase(sexp:list(X),X).
test(list,[setup(atom_chars('(1 . 2)',X))]) :- phrase(sexp:list(X),X).
test(list,[setup(atom_chars('[1 . 2]',X))]) :- phrase(sexp:list(X),X).
test(list,[setup(atom_chars('(1.2 #f "構文". 2)',X))]) :- phrase(sexp:list(X),X).
test(list,[setup(atom_chars('[9 () #(). 2]',X))]) :- phrase(sexp:list(X),X).

test(vector,[setup(atom_chars('#(0 (2 2 2 2) "Anna")',X))]) :- phrase(sexp:vector(X),X).

test(bytevector,[setup(atom_chars('#vu8(2 24 123)',X))]) :- phrase(sexp:bytevector(X),X).

test(symbol,[setup(atom_chars(':keyword',X))]) :- phrase(sexp:symbol(X),X).

test(datum,[setup(atom_chars('a',X))]) :- phrase(sexp:datum(X),X).
test(datum,[setup(atom_chars('1',X))]) :- phrase(sexp:datum(X),X).
test(datum,[setup(atom_chars('#t',X))]) :- phrase(sexp:datum(X),X).
test(datum,[setup(atom_chars('(rep 1 1 3 :caption "濁点が抜けている" :p 78 :quote "スライドクラス" :fix  "スライドグラス")',X))]) :- phrase(sexp:datum(X),X).

%% Translation of Term

test(term_to_sexp,[setup(atom_chars('#t',X))]) :- term_to_sexp(true,X).
test(term_to_sexp,[setup(atom_chars('#f',X))]) :- term_to_sexp(false,X).
test(term_to_sexp,[setup(atom_chars('abc',X))]) :- term_to_sexp(abc,X).
test(term_to_sexp,[setup(atom_chars('1',X))]) :- term_to_sexp(1,X).
test(term_to_sexp,[setup(atom_chars('1023',X))]) :- term_to_sexp(1023,X).
test(term_to_sexp,[setup(atom_chars('()',X))]) :- term_to_sexp([],X).
test(term_to_sexp,[setup(atom_chars('(1 2 3)',X))]) :- term_to_sexp([1,2,3],X).
test(term_to_sexp,[setup(atom_chars('(abc 2 3)',X))]) :- term_to_sexp(abc(2,3),X).

%% sexp_write_string

test(sexp_write_string,[setup(atom_chars('',X))]) :-
    with_output_to(atom(A),sexp:sexp_write_string(X)),
    A = '""'.
test(sexp_write_string,[setup(atom_chars('abc def',X))]) :-
    with_output_to(atom(A),sexp:sexp_write_string(X)),
    A = '"abc def"'.
test(sexp_write_string) :-
    with_output_to(atom(A),sexp:sexp_write_string('')),
    A = '""'.
test(sexp_write_string) :-
    with_output_to(atom(A),sexp:sexp_write_string('""')),
    A = '"\\"\\""'.
test(sexp_write_string) :-
    with_output_to(atom(A),sexp:sexp_write_string('\n\r')),
    A = '"\\n\\r"'.
test(sexp_write_string) :-
    with_output_to(atom(A),sexp:sexp_write_string('あ')),
    A = '"あ"'.
test(sexp_write_string) :-
    with_output_to(atom(A),sexp:sexp_write_string('foobar')),
    A = '"foobar"'.

:- end_tests(sexp).
