%% -*- coding: utf-8 -*-
:- begin_tests(typo).
:- use_module(typo).
:- use_module(isbn).
:- encoding(utf8).

test(rep) :- forall((rep(_,_,_,Rep),option(isbn10(ISBN10),Rep)),
                    (atom_chars(ISBN10,L10),phrase(isbn10(_),L10))).
test(rep) :- forall((rep(_,_,_,Rep),option(isbn13(ISBN13),Rep)),
                    (atom_chars(ISBN13,L13),phrase(isbn13(_),L13))).

test(declaration) :- declaration(account, password, [optional(true), default('')]).
test(declaration) :- declaration(account, password_repeat, [optional(true), default('')]).
test(declaration) :- declaration(bib, title, [optional(true), default('')]).
test(declaration) :- declaration(bib, id, [integer]).

test(currval) :- forall(currval(Name,N),(database(Name/L),L1 is L+1,length(X,L1),X = [Name,Id|_],T =.. X,Term =.. [:,typo,T],forall(Term,Id =< N))).
test(nextval, [setup(assert(currval(foo, 42))), cleanup(retractall(currval(foo, _)))]) :-
    nextval(foo, 43).

test(shared_review) :- forall(shared_review(Id),review(Id,_,_,_)).
test(recent_review) :- forall(recent_review(Id),shared_review(Id)).
test(review_history) :- forall(review(_,U,B,X),review_history(U,B,X)).

test(shared_rep) :- forall(shared_rep(Id),rep(Id,_,_,_)).
test(recent_rep) :- forall(recent_rep(Id),shared_rep(Id)).

test(updated_at,[setup(rep(12,_,_,Rep))]) :- updated_at(Rep,date(2008,2,10,1,2,26.1345,-32400,'JST',false)).
test(order_by_updated_at) :-
    get_time(X),stamp_date_time(X,DX,local),get_time(Y),stamp_date_time(Y,DY,local),
    order_by_updated_at(asc,<,[updated_at(DX)],[updated_at(DY)]),
    order_by_updated_at(desc,>,[updated_at(DX)],[updated_at(DY)]).
test(order_by_updated_at) :-
    get_time(X),stamp_date_time(X,DX,local),
    order_by_updated_at(asc,>,[updated_at(DX)],[]),
    order_by_updated_at(desc,<,[updated_at(DX)],[]).
test(order_by_updated_at) :-
    get_time(X),stamp_date_time(X,DX,local),
    order_by_updated_at(asc,<,[],[updated_at(DX)]),
    order_by_updated_at(desc,>,[],[updated_at(DX)]).
test(order_by_updated_at) :-
    order_by_updated_at(asc,=,[],[]),
    order_by_updated_at(desc,=,[],[]).

test(nickname,[setup(user(1,User))]) :- nickname(User,tabe).

test(mail_address,[setup(user(1,User))]) :- mail_address(User,'tabe@fixedpoint.jp').

test(reserved_nickname) :- forall((user(_,User),nickname(User,Nickname)),reserved_nickname(Nickname)).
test(reserved_nickname) :- reserved_nickname('tabe').
test(reserved_nickname,[fail]) :- reserved_nickname('humpty dumpty').

test(reserve_nickname,[fail]) :- reserve_nickname('tabe').
test(reserve_nickname) :- Nickname = 'humpty dumpty',reserve_nickname(Nickname),retract(reserved_nickname(Nickname)).

test(title,[setup(bib(1,Bib))]) :- title(Bib,'Emacs Lisp リファレンスマニュアル').

test(notify,[setup(bookshelf(1,_,_,Bookshelf))]) :- notify(Bookshelf,mail).

test(caption,[setup(rep(1,_,_,Rep))]) :- caption(Rep,'濁点が抜けている').

test(body,[setup(review(1,_,_,Review))]) :- body(Review,'"マイナモード"が気になる。').

test(share,[setup(share(rep(1))),cleanup(unshare(rep(1)))]) :- shared_rep(1),recent_rep(1).
test(share,[setup(share(review(2))),cleanup(unshare(review(2)))]) :- shared_review(2),recent_review(2).

test(yes) :- yes('Yes', false).
test(yes) :- yes('yes', false).
test(yes) :- yes('y', false).

test(pages,[setup(atom_chars('-',X))]) :- phrase(pages(-),X).
test(pages,[setup(atom_chars('102-104',X))]) :- phrase(pages(102-104),X).
test(pages,[setup(atom_chars('45f',X))]) :- phrase(pages(45-f),X).
test(pages,[setup(atom_chars('28ff',X))]) :- phrase(pages(28-ff),X).

:- end_tests(typo).
