%% -*- coding: utf-8 -*-
:- module(typo_html,
          [ ordered_list/3,
            unordered_list/3,
            typo_post/3,
            css/3,
            js/3,
            table/3,
            logo/2,
            copyright/2,
            form/4,
            page_title/2,
            diff/4,
            syn_html/3,
            recent_reps/2,
            recent_reviews/2
          ]).
:- use_module(library('http/html_write')).
:- use_module(lcs).
:- use_module(typo).
:- use_module(typo_url).
:- encoding(utf8).

ordered_list([]) --> [].
ordered_list(List) -->
    html_begin(ol),
    list_items(List),
    html_end(ol).
unordered_list([]) --> [].
unordered_list(List) -->
    html_begin(ul),
    list_items(List),
    html_end(ul).
list_items([]) --> [].
list_items([Item|Rest]) -->
    html_begin(li),
    html(Item),
    html_end(li),
    list_items(Rest).

typo_post(Term) -->
    { Term =.. [Key,Value] },
    html_post(Key,Value).

%% html components placed unique locations
css(URL) -->
    html_post(head,link([ type('text/css'),rel('stylesheet'),href(URL) ])).
js(Content) -->
    { is_list(Content) },
    html(script([type('text/javascript')],\Content)),!.
js(Content) --> js([Content]).

%% html components defined by DCG
table_columns([]) --> [].
table_columns([Column|Rest]) -->
    html_begin(td),
    html([Column]),
    html_end(td),
    table_columns(Rest).
table_rows([]) --> [].
table_rows([Row|Rest]) -->
    html_begin(tr),
    table_columns(Row),
    html_end(tr),
    table_rows(Rest).
table(Rows) -->
    html_begin(table),
    html_begin(tbody),
    table_rows(Rows),
    html_end(tbody),
    html_end(table).

logo --> html(image([src('/images/typo.png')],[])).

copyright --> html(address([&(copy),' 2008 fixedpoint.jp'])).

form(Closure,Content) -->
    { handler(URL,Closure) },
    html(form([method('POST'),action(URL)],Content)).

page_title(root,title('Typo')) :- !.
page_title(X,title(['Typo -- ',X])).

link(Closure,a(href(Location),Closure)) :-
    handler(Location,Closure).
link(Closure,Search,a(href(Path),Closure)) :-
    typo_url:path(Closure,Search,Path).

link(Closure,Search,Content) -->
    { typo_url:path(Closure,Search,Path) },
    html(a(href(Path),Content)).

input_ticket(Ticket) -->
    html(input([type(hidden),name(ticket),value(Ticket)])).
input_yes_or_no -->
    html([input([type(submit),name(no),value('No')]),
          input([type(submit),name(yes),value('Yes')])
         ]).

diff_entity(' ',nbsp).

a_only('\n',[X,Y],[[br([])|X],Y]) :- !.
a_only(C,[X,Y],[[del('&'(E))|X],Y]) :- diff_entity(C,E),!.
a_only(C,[X,Y],[[del(C)|X],Y]).
b_only('\n',[X,Y],[X,[br([])|Y]]) :- !.
b_only(C,[X,Y],[X,[ins('&'(E))|Y]]) :- diff_entity(C,E),!.
b_only(C,[X,Y],[X,[ins(C)|Y]]).
both('\n',[X,Y],[[br([])|X],[br([])|Y]]) :- !.
both(C,[X,Y],[['&'(E)|X],['&'(E)|Y]]) :- diff_entity(C,E),!.
both(C,[X,Y],[[C|X],[C|Y]]).

diff(A,B) -->
    { lcs_fold(typo_html:a_only,typo_html:b_only,typo_html:both,[[],[]],A,B,[X,Y]),
      reverse(X,RX),
      reverse(Y,RY) },
    html_begin(div),
    html_begin(blockquote),
    html(RX),
    html_end(blockquote),
    html_end(div),
    html_begin(div),
    html_begin(blockquote),
    html(RY),
    html_end(blockquote),
    html_end(div).

syn_html([]) --> [].
syn_html(X) -->
    html_begin(ul(id(syns))),
    syns_html(X),
    html_end(ul),
    js('Event.observe(window,"load",function(e){Sortable.create("syns",{dropOnEmpty:true,handle:"handle",constraint:false});});').

syns_html([]) --> [].
syns_html([X|Rest]) -->
    { strip_module(X,_,syn(Id,UserId,RepId,Syn,Options)),
      atom_concat('syn_',Id,DomId),
      format(atom(JS),'$(\'~a\').toggle();',[DomId]),
      typo:user(UserId,User),
      typo:nickname(User,Nickname),
      typo:rep(RepId,_,_,Rep),
      typo:quote(Rep,Quote),
      ( typo:fix(Syn,Fix) ->
        atom_chars(Quote,QList),
        atom_chars(Fix,FList)
      ; QList = [],
        FList = []),
      ( typo:body(Syn,Body)
      ; Body = ''),
      typo:acks(Id,Acks)
    },
    html_begin(li),
    html(div([class(handle)],[span([class(o),onclick(JS)],'o')])),
    html_begin(div(id(DomId),class(syn))),
    html([Nickname,'> ',Body,div([\diff(QList,FList)])]),
    acks_html(Acks,Id,Options),
    html_end(div),
    html_end(li),
    syns_html(Rest).

acks_html(Acks,Id,Options) -->
    html_begin(ul),
    ack_html(Acks),
    ack_form(Id,Options),
    html_end(ul).

ack_html([]) --> [].
ack_html([ack(_Id,UserId,_SynId,Ack)|Rest]) -->
    { typo:user(UserId,User),
      typo:nickname(User,Nickname),
      typo:body(Ack,Body) },
    html(div([Nickname,'> ',Body])),
    ack_html(Rest).

ack_form(Id,Options) -->
    { typo:nickname(Options,Nickname) },
    html(div([\form(send_ack,
                    [input([type(hidden),name(id),value(Id)]),
                     Nickname,'> ',
                     input([type(text),name(body),value('')]),
                     input([type(submit),name(submit),value('send')])])])).
ack_form(_,_) --> [].

recent_reps -->
    { findall(Id,typo:recent_rep(Id),Ids) },
    html(h3('Recent Reps')),
    html(ul(\recent_reps(Ids))).
recent_reps([]) --> [].
recent_reps([Id|Rest]) -->
    recent_rep(Id),
    recent_reps(Rest).
recent_rep(Id) -->
    { typo:rep(Id,_,_,Rep),
      typo:caption(Rep,Caption) },
    html(li(\link(rep,[id=Id],Caption))),!.
recent_rep(_) --> [].

recent_reviews -->
    { findall(Id,typo:recent_review(Id),Ids) },
    html(h3('Recent Reviews')),
    html(ul(\recent_reviews(Ids))).
recent_reviews([]) --> [].
recent_reviews([Id|Rest]) -->
    recent_review(Id),
    recent_reviews(Rest).
recent_review(Id) -->
    { typo:review(Id,_,BibId,Review),
      typo:body(Review,Body) },
    html(li(\link(bib,[id=BibId],Body))),!.
recent_review(_) --> [].
