%% -*- coding: utf-8 -*-
:- module(typo_sql,[typo_sql_insert/1,
                    typo_sql_delete/1,
                    typo_sql_update/1]).
:- use_module(sexp).
:- use_module(typo).
:- encoding(utf8).

typo_sql_table(user,users).
typo_sql_table(bib,bibs).
typo_sql_table(bookshelf,bookshelves).
typo_sql_table(rep,reps).
typo_sql_table(syn,syns).
typo_sql_table(ack,acks).
typo_sql_table(review,reviews).

typo_sql_form(user(Id,_),id(positive_integer(Id))).
typo_sql_form(user(_,Options),nickname(string(Nickname))) :- option(nickname(Nickname),Options).
typo_sql_form(user(_,Options),mail_address(string(MailAddress))) :- option(mail_address(MailAddress),Options).
typo_sql_form(user(_,Options),password(string(Password))) :- option(password(Password,_),Options).
typo_sql_form(user(_,Options),password_algorithm(string(Algorithm))) :- option(password(_,O),Options),option(algorithm(Algorithm),O).

typo_sql_form(bib(Id,_),id(positive_integer(Id))).
typo_sql_form(bib(_,Options),title(string(Title))) :- option(title(Title),Options).
typo_sql_form(bib(_,Options),isbn10(string(ISBN))) :- memberchk(isbn(10,ISBN),Options).
typo_sql_form(bib(_,Options),isbn13(string(ISBN))) :- memberchk(isbn(13,ISBN),Options).
typo_sql_form(bib(_,Options),edition(positive_integer(Edition))) :- option(edition(Edition),Options).
typo_sql_form(bib(_,Options),impression(positive_integer(Impression))) :- option(impression(Impression),Options).
typo_sql_form(bib(_,Options),image_src(string(URL))) :- option(image(Image),Options),option(src(URL),Image).
typo_sql_form(bib(_,Options),image_width(positive_integer(Width))) :- option(image(Image),Options),option(width(Width),Image).
typo_sql_form(bib(_,Options),image_height(positive_integer(Height))) :- option(image(Image),Options),option(height(Height),Image).

typo_sql_form(bookshelf(Id,_,_,_),id(positive_integer(Id))).
typo_sql_form(bookshelf(_,UserId,_,_),user_id(positive_integer(UserId))).
typo_sql_form(bookshelf(_,_,BibId,_),bib_id(positive_integer(BibId))).
typo_sql_form(bookshelf(_,_,_,Options),notify(string(Notify))) :- option(notify(Notify),Options).

typo_sql_form(rep(Id,_,_,_),id(positive_integer(Id))).
typo_sql_form(rep(_,UserId,_,_),user_id(positive_integer(UserId))).
typo_sql_form(rep(_,_,BibId,_),bib_id(positive_integer(BibId))).
typo_sql_form(rep(_,_,_,Rep),caption(string(Caption))) :- option(caption(Caption),Rep).
typo_sql_form(rep(_,_,_,Rep),p(string(P))) :- option(p(P),Rep).
typo_sql_form(rep(_,_,_,Rep),quote(string(Quote))) :- option(quote(Quote),Rep).

typo_sql_form(syn(Id,_,_,_),id(positive_integer(Id))).
typo_sql_form(syn(_,UserId,_,_),user_id(positive_integer(UserId))).
typo_sql_form(syn(_,_,RepId,_),rep_id(positive_integer(RepId))).
typo_sql_form(syn(_,_,_,Syn),fix(string(Fix))) :- option(fix(Fix),Syn).
typo_sql_form(syn(_,_,_,Syn),body(string(Body))) :- option(body(Body),Syn).

typo_sql_form(ack(Id,_,_,_),id(positive_integer(Id))).
typo_sql_form(ack(_,UserId,_,_),user_id(positive_integer(UserId))).
typo_sql_form(ack(_,_,SynId,_),syn_id(positive_integer(SynId))).
typo_sql_form(ack(_,_,_,Ack),body(string(Body))) :- option(body(Body),Ack).

typo_sql_form(review(Id,_,_,_),id(positive_integer(Id))).
typo_sql_form(review(_,UserId,_,_),user_id(positive_integer(UserId))).
typo_sql_form(review(_,_,BibId,_),bib_id(positive_integer(BibId))).
typo_sql_form(review(_,_,_,Review),body(string(Body))) :- option(body(Body),Review).

typo_sql_format(Field) :-
    Field =.. [Name,Column],
    Column =.. [Type,Value],
    format(' :~a ',[Name]),
    typo_sql_format(Type,Value).

typo_sql_format(string,Value) :- !,sexp_write_string(Value).
typo_sql_format(positive_integer,Value) :-
    integer(Value),Value > 0,!,
    write(Value).
typo_sql_format(positive_integer,_) :- !,write(0).
typo_sql_format(_,Value) :- write(Value).

typo_sql_insert(Term) :-
    Term =.. [N|_],
    typo_sql_table(N,Name),
    write('(typo-sql-insert '),
    sexp_write_string(Name),
    forall(typo_sql_form(Term,Field),typo_sql_format(Field)),
    write(')').

typo_sql_delete(Term) :-
    Term =.. [N,Id|_],
    typo_sql_table(N,Name),
    write('(typo-sql-delete '),
    sexp_write_string(Name),
    format(' ~d)',[Id]).

typo_sql_update(Term) :-
    Term =.. [N,Id|_],
    typo_sql_table(N,Name),
    write('(typo-sql-update '),
    sexp_write_string(Name),
    write(' ('),
    forall(typo_sql_form(Term,Field),(functor(Field,id,1),!;typo_sql_format(Field))),
    format(') ~d)',[Id]).

:- encoding(utf8).
