:- module(typod,
          [ server/1,
            stop_server/1,
            bs/3,
            maplist_bs/3,
            give_ticket/2,
            take_ticket/2
          ]).
:- use_module(library('http/http_client')).
:- use_module(library('http/thread_httpd')).
:- use_module(library('http/http_dispatch')).
:- use_module(library('http/html_write')).
:- use_module(library('http/http_session')).
:- use_module(library('http/http_parameters')).
:- use_module(typo).
:- use_module(typo_aws).
:- use_module(typo_dbi).
:- use_module(typo_interaction).
:- use_module(typo_html).
:- use_module(typo_mail).
:- use_module(typo_pigeon).
:- use_module(typo_url).
:- encoding(utf8).

:- dynamic log/1,ticket/3.

logger(Request) :- assert(log(Request)).

give_ticket(Ticket,Stamp) :-
    R1 is random(65536),
    R2 is random(65536),
    R3 is random(65536),
    R4 is random(65536),
    format(atom(T),
           '~`0t~16r~4|-~`0t~16r~9|-~`0t~16r~14|-~`0t~16r~19|',
           [R1,R2,R3,R4]),
    ( ticket(_,T,_) ->
      give_ticket(Ticket,Stamp) % try again
    ; Ticket = T,
      http_session_id(SessionId),
      assert(ticket(SessionId,Ticket,Stamp))
    ).

take_ticket(Ticket,Stamp) :-
    http_session_id(SessionId),
    retract(ticket(SessionId,Ticket,Stamp)).

share_and_notify(Term) :-
    typo:share(Term),
    Term =.. [Name,Id,FromUserId,BibId|_],
    mail_sender(Sender),
    forall((typo:bookshelf(_,UserId,BibId,Bookshelf),
            FromUserId =\= UserId,
            typo:notify(Bookshelf,mail),
            typo:user(UserId,User),
            typo:nickname(User,Nickname),
            typo:mail_address(User,MailAddress)
           ),
           ( mail_template(notify(Name,Id,Nickname),Subject,Body),
             typo_pigeon(MailAddress,Sender,Subject,Body),!
           ; true
           )).

menu(root,forward(home)).
menu(join,back(root)).
menu(welcome,back(home)).
menu(home,back(root)).
menu(home,forward(bookshelf)).
menu(home,forward(desk)).
menu(home,forward(window)).
menu(bookshelf,back(home)).
menu(bookshelf,forward(put_on)).
menu(put_on,back(bookshelf)).
menu(put_off,back(bookshelf)).
menu(write_review,back(bookshelf)).
menu(share_review,back(bookshelf)).
menu(desk,back(home)).
menu(create_rep,back(desk)).
menu(create_rep,back(bookshelf)).
menu(share_rep,back(desk)).
menu(share_rep,back(bookshelf)).
menu(bib,back(home)).
menu(rep,back(home)).
menu(window,back(home)).
menu(window,forward(account)).
menu(account,back(window)).
menu(account,forward(modify_password)).
menu(account,forward(modify_mail_address)).
menu(account,forward(leave)).
menu(modify_mail_address,back(account)).
menu(modify_password,back(account)).
menu(leave,back(account)).

static_location(Type,Name,Path,Location) :-
    concat_atom([Type,'/',Name],Path),
    atom_concat('/',Path,Location).

static_handler(Type,Name) :-
    static_location(Type,Name,Path,Location),
    http_handler(Location,http_reply_file(Path,[]),[]).

server(Port) :-
    forall(handler(Location,Closure),http_handler(Location,Closure,[])),
    forall(css(CSS),static_handler(stylesheets,CSS)),
    forall(js(JS),static_handler(javascripts,JS)),
    forall(image(Image),static_handler(images,Image)),
    http_set_session_options([timeout(3600),cookie(typo_session)]),
    http_server(http_dispatch,
                [ port(Port),
                  timeout(20),
                  after(logger),
                  workers(3)
                ]).

stop_server(Port) :- http_stop_server(Port,[]).

go_to(Closure) :-
    handler(Location,Closure),
    redirect_to(Location).
redirect_to(Location) :-
    throw(http_reply(moved_temporary(Location))).
redirect_to(Closure,Search) :-
    path(Closure,Search,Location),
    redirect_to(Location).

deny_user :-
    http_session_data(user(_)),!,
    redirect_to(/).

current_user(Id,User) :-
    http_session_data(user(Id)),
    typo:user(Id,User).

method(Method,Request) :- memberchk(method(Method),Request).

bs(Closure,Term,Result) :-
    SubTerm =.. [Closure,Term],
    Result =.. [\,SubTerm].

maplist_bs(Closure,List,Result) :- maplist(bs(Closure),List,Result).

frame(Closure,Content,Body) :-
    current_user(_,User),
    nickname(User,Name),
    %% 
    findall(B,menu(Closure,back(B)),Back),
    findall(F,menu(Closure,forward(F)),Forward),
    maplist(typo_html:link,Back,BLinks),
    maplist(typo_html:link,Forward,FLinks),
    findall(['[',L,']'],member(L,BLinks),X),
    findall(['[',L,']'],member(L,FLinks),Y),
    flatten(X,XX),
    flatten(Y,YY),
    %%
    Body = [ div([id(status),class(right)],['Hi, ',Name,' [',a(href('/logout'),logout) ,']' ]),
             \logo,
             div([ div(id(back),span(XX)),
                   div(id(menu),Closure),
                   div(id(forward),span(YY))
                 ]),
             div([id(content)],Content),
             \copyright
           ],!.
frame(_,Content,Body) :-
    Body = [ \js('Event.observe(window,"load",function(e){$("login_nickname").focus();});'),
             div([id(status),class(right)],
                 [\form(login,
                        [div(['nickname ',input([id(login_nickname),type(text),name(nickname)]),' ',
                              'password ',input([type(password),name(password)]),
                              input([type(submit),value('login')],[])])
                        ])]),
              \logo,
             div([id(content)],
                 [ div(class(development),
                       [div('This is a kind of web application for sharing typos. Still in development.'),
                        div(['You can ',a(href('/join'),join),' us.'])
                       ])
                 | Content]),
             \copyright
           ].

reply_typo_page(Closure,Content) :-
    findall(\html_post(css,link([type('text/css'),rel('stylesheet'),href(Location)])),
            (css(CSS),static_location(stylesheets,CSS,_,Location)),
            CSS_Posts),
    findall(\html_post(js_head,script([type('text/javascript'),src(Location)],'')),
            (member(JS,['prototype.js','scriptaculous.js','typo.js']),static_location(javascripts,JS,_,Location)),
            JS_Posts),
    append(CSS_Posts,JS_Posts,Posts),
    reply_typo_page(Closure,Content,Posts).
reply_typo_page(Closure,Content,Options) :-
    page_title(Closure,Title),
    frame(Closure,Content,Body),
    reply_html_page([ \html_receive(css),
                      Title,
                      \html_receive(js_head)
                    | Options ],
                    [ \html_receive(js_body)
                    | Body]).

root(_) :- reply_typo_page(root,[\recent_reps,\recent_reviews]).

login(_) :- deny_user,!.
login(Request) :-
    method(post,Request),
    http_parameters(Request,
                    [nickname(Nickname),password(Password)],
                    [attribute_declarations(declaration(account))]),
    validate(nickname,Nickname),
    validate(password,Password),
    fetch_user(Nickname,Password,Id),
    http_session_assert(user(Id)),
    redirect_to(/).
login(_) :- redirect_to(/).

logout(_) :-
    http_session_retractall(user(_)),!,
    redirect_to(/).
logout(_) :- redirect_to(/).

join(_) :- deny_user,!.
join(Request) :-
    method(post,Request),
    http_parameters(Request,
                    [ nickname(Nickname),
                      password(Password),
                      password_repeat(Password),
                      mail_address(MailAddress),
                      mail_address_repeat(MailAddress)
                    ],
                    [attribute_declarations(declaration(account))]),
    ( validate(nickname,Nickname),
      \+ reserved_nickname(Nickname),
      validate(password,Password),
      validate(mail_address,MailAddress) ->
      reserve_nickname(Nickname),
      give_ticket(Ticket,join(Nickname,Password,MailAddress)),
      mail_sender(Sender),
      mail_template(welcome(Nickname,Ticket),Subject,Body),
      typo_pigeon(MailAddress,Sender,Subject,Body),
      reply_typo_page(join,[div(['Our pigeon will send an e-mail to you.\n',
                                 'You can confirm your request with it.\n',
                                 'Thank you.'])])
    ; template(join(Nickname))
    ).
join(_) :- template(join('')).

welcome(Name,Password,MailAddress,Id) :-
    Option = [algorithm(sha1)],
    sha_hash(Password,Hash,Option),
    string_to_list(S,Hash),
    base64(S,Base64),
    nextval(user,Id),
    typo_dbi_assert(user(Id,[nickname(Name),mail_address(MailAddress),password(Base64,Option)])).

welcome(Request) :-
    method(get,Request),
    http_parameters(Request,[ticket(Ticket)],[attribute_declarations(declaration(welcome))]),
    take_ticket(Ticket,join(Nickname,Password,MailAddress)),
    welcome(Nickname,Password,MailAddress,Id),
    http_session_assert(user(Id)),
    reply_typo_page(welcome,[div([ 'Welcome! ',Nickname])]).
welcome(_) :- redirect_to(/).

home(Request) :-
    method(get,Request),
    current_user(_,_),
    reply_typo_page(home,[div([image([src('/images/home.png')],[])])]).
home(_) :- redirect_to(/).

spine(Id,li([span(class(brook),['[',a(href(PathCreateRep),create_rep),']']),' ',a(href(PathBib),Title)])) :-
    typo:bib(Id,Bib),
    typo:title(Bib,Title),
    path(bib,[id=Id],PathBib),
    path(create_rep,[id=Id],PathCreateRep).

bookshelf(Request) :-
    method(get,Request),
    current_user(UserId,_),
    findall(BibId,typo:bookshelf(_,UserId,BibId,_),BibIds),
    maplist(spine,BibIds,Spines),
    reply_typo_page(bookshelf,[image([src('/images/bookshelf.png')],[]),ul(Spines)]).
bookshelf(_) :- redirect_to(/).

put_on(title(Title)) --> html_post(title,input([type(text),name(title),value(Title)])).
put_on(isbn(ISBN)) --> html_post(isbn,input([type(text),name(isbn),value(ISBN)])).
put_on(edition(Edition)) --> html_post(edition,input([type(text),name(edition),value(Edition)])).
put_on(impression(Impression)) --> html_post(impression,input([type(text),name(impression),value(Impression)])).
put_on(image(Image)) --> html_post(image,image(Image,[])).
put_on(force(Force)) --> html_post(force,input([type(hidden),name(force),value(Force)])).
put_on(Term) --> typo_post(Term).

%% put on a new bib on one's bookshelf.
put_on(Request) :-
    method(post,Request),
    current_user(UserId,_),
    Input = [edition(_),force(_),impression(_),isbn(_),title(_)], % sorted
    http_parameters(Request,Input,[attribute_declarations(declaration(bib))]),
    typo_interaction(put_on,Input,Action,Output),
    ( Action == next ->
      option(title(Title),Output),
      nextval(bib,BibId),
      nextval(bookshelf,BookshelfId),
      typo_dbi_assert(bib(BibId,Output)),
      typo_dbi_assert(bookshelf(BookshelfId,UserId,BibId,[])),
      reply_typo_page(put_on,['New bib `',Title,'\' has been put on your bookshelf.'])
    ; Action == retry ->
      template(put_on(Output))
    ; template(put_on([message('Hint: try to specify the one with its ISBN.')|Input]))
    ).
put_on(_) :-
    current_user(_,_),
    template(put_on).
put_on(_) :- redirect_to(/).

%% put off an existing bib on one's bookshelf.
put_off(Request) :-
    method(post,Request),
    current_user(UserId,_),
    http_parameters(Request,[ticket(Ticket),id(BibId),yes(Yes),no(No)],[attribute_declarations(declaration(bib))]),
    take_ticket(Ticket,put_off(BibId)),
    ( yes(Yes,No),
      retractall(typo:bookshelf(_,UserId,BibId,_)),
      typo:bib(BibId,Bib),
      typo:title(Bib,Title) ->
      reply_typo_page(put_off,['The bib `',Title,'\' has been put off on your bookshelf.'])
    ; redirect_to(bib,[id=BibId])
    ).
put_off(Request) :-
    method(get,Request),
    current_user(UserId,_),
    http_parameters(Request,[id(BibId)],[attribute_declarations(declaration(bib))]),
    typo:bookshelf(_,UserId,BibId,_),
    typo:bib(BibId,Bib),
    typo:title(Bib,Title),
    give_ticket(Ticket,put_off(BibId)),
    reply_typo_page(put_off,
                    [ div(['Would you like to put off `',Title,'\' on your bookshelf really?']),
                      \form(put_off,
                            [ \input_ticket(Ticket),
                              input([type(hidden),name(id),value(BibId)]),
                              input([type(submit),name(no),value('No')]),
                              input([type(submit),name(yes),value('Yes')])
                            ])
                    ]).
put_off(_) :- redirect_to(/).

%% write or update the review of a bib.
write_review(Request) :-
    method(post,Request),
    current_user(UserId,_),
    http_parameters(Request,[ticket(Ticket),id(BibId),body(Body)],[attribute_declarations(declaration(review))]),
    take_ticket(Ticket,write_review(BibId)),
    ( nextval(review,ReviewId),
      get_time(T),
      stamp_date_time(T,D,local),
      retractall(typo:review(_,UserId,BibId,_)),
      typo_dbi_assert(review(ReviewId,UserId,BibId,[body(Body),updated_at(D)])),
      assert(typo:review_history(UserId,BibId,[body(Body),updated_at(D)])) ->
      redirect_to(share_review,[id=ReviewId])
    ; redirect_to(bib,[id=BibId])
    ).
write_review(Request) :-
    method(get,Request),
    current_user(UserId,_),
    http_parameters(Request,[id(BibId)],[attribute_declarations(declaration(bib))]),
    typo:bookshelf(_,UserId,BibId,_),
    typo:bib(BibId,Bib),
    typo:title(Bib,Title),
    ( typo:review(_,UserId,BibId,Review) ->
      typo:body(Review,Body)
    ; Body = ''
    ),
    give_ticket(Ticket,write_review(BibId)),
    reply_typo_page(write_review,
                    [div(['If you would like to write your review of `',Title,'\', let me know:']),
                     \form(write_review,
                           [textarea([name(body)],Body),
                            \input_ticket(Ticket),
                            input([type(hidden),name(id),value(BibId)]),
                            input([type(submit),value(submit)])
                           ])
                    ]).
write_review(_) :- redirect_to(/).

%% share the review of a bib.
share_review(Request) :-
    method(post,Request),
    current_user(UserId,_),
    http_parameters(Request,[ticket(Ticket),id(ReviewId),yes(Yes),no(No)],[attribute_declarations(declaration(review))]),
    take_ticket(Ticket,share_review(ReviewId)),
    typo:review(ReviewId,UserId,BibId,_),
    ( yes(Yes,No),
      share_and_notify(review(ReviewId,UserId,BibId)) ->
      reply_typo_page(share_review,['The review has been shared.'])
    ; redirect_to(bib,[id=BibId])
    ).
share_review(Request) :-
    method(get,Request),
    current_user(UserId,_),
    http_parameters(Request,[id(ReviewId)],[attribute_declarations(declaration(review))]),
    typo:review(ReviewId,UserId,_,_),
    give_ticket(Ticket,share_review(ReviewId)),
    reply_typo_page(share_review,
                    [image([src('/images/share.png')],[]),
                     div('Would you like to share the review?'),
                     \form(share_review,
                           [\input_yes_or_no,
                            \input_ticket(Ticket),
                            input([type(hidden),name(id),value(ReviewId)])
                           ])
                    ]).
share_review(_) :- redirect_to(/).

dt_and_dd(X-Y,[dt(X)|Z]):- findall(dd(DD),member(DD,Y),Z).

dl(X,dl(List)) :-
    maplist(dt_and_dd,X,Y),
    flatten(Y,List).

rep_postit(Id-Rep,a(href(Path),Caption)) :-
    typo:caption(Rep,Caption),
    path(rep,[id=Id],Path).

postit(Id-RPairs,[a(href(PathBib),Title),span(class(brook),[' [',a(href(PathCreateRep),create_rep),']'])]-LPairs) :-
    typo:bib(Id,Bib),
    typo:title(Bib,Title),
    path(bib,[id=Id],PathBib),
    path(create_rep,[id=Id],PathCreateRep),
    maplist(rep_postit,RPairs,LPairs).

desk(Request) :-
    method(get,Request),
    current_user(UserId,_),
    findall(BibId-(RepId-Rep),typo:rep(RepId,UserId,BibId,Rep),X),
    group_pairs_by_key(X,Y),
    maplist(postit,Y,Z),
    dl(Z,DL),
    reply_typo_page(desk,[image([src('/images/desk.png'),width(416),height(116)],[]),DL]).
desk(_) :- redirect_to(/).

create_rep(Request) :-
    current_user(UserId,_),
    method(post,Request),
    http_parameters(Request,[id(BibId),caption(Caption),p(P),quote(Quote),fix(Fix)],[attribute_declarations(declaration(rep))]),
    ( nextval(rep,RepId),
      nextval(syn,SynId),
      get_time(T),
      stamp_date_time(T,D,local),
      typo_dbi_assert(rep(RepId,UserId,BibId,[caption(Caption),p(P),quote(Quote),updated_at(D),created_at(D)])),
      typo_dbi_assert(syn(SynId,UserId,RepId,[fix(Fix),updated_at(D),created_at(D)])) ->
      redirect_to(share_rep,[id=RepId])
    ; template(create_rep(BibId,[caption(Caption),p(P),quote(Quote),fix(Fix)]))
    ).
create_rep(Request) :-
    current_user(UserId,_),
    http_parameters(Request,[id(BibId)],[attribute_declarations(declaration(rep))]),
    typo:bookshelf(_,UserId,BibId,_),
    template(create_rep(BibId,[caption(''),p(''),quote(''),fix('')])).
create_rep(_) :- redirect_to(/).

%% share the rep of a bib.
share_rep(Request) :-
    method(post,Request),
    current_user(UserId,_),
    http_parameters(Request,[ticket(Ticket),id(RepId),yes(Yes),no(No)],[attribute_declarations(declaration(rep))]),
    take_ticket(Ticket,share_rep(RepId)),
    typo:rep(RepId,UserId,BibId,_),
    ( yes(Yes,No),
      share_and_notify(rep(RepId,UserId,BibId)) ->
      reply_typo_page(share_rep,['The rep has been shared.'])
    ; redirect_to(bib,[id=BibId])
    ).
share_rep(Request) :-
    method(get,Request),
    current_user(UserId,_),
    http_parameters(Request,[id(RepId)],[attribute_declarations(declaration(rep))]),
    typo:rep(RepId,UserId,_,_),
    give_ticket(Ticket,share_rep(RepId)),
    reply_typo_page(share_rep,
                    [image([src('/images/share.png')],[]),
                     div('Would you like to share the rep?'),
                     \form(share_rep,
                           [\input_yes_or_no,
                            \input_ticket(Ticket),
                            input([type(hidden),name(id),value(RepId)])
                           ])
                    ]).
share_rep(_) :- redirect_to(/).

send_syn(Request) :-
    method(post,Request),
    current_user(UserId,_),
    http_parameters(Request,[id(RepId),fix(Fix),body(Body)],[attribute_declarations(declaration(syn))]),
    typo:rep(RepId,_,_,Rep),
    typo:quote(Rep,Quote),
    ( Quote == Fix ->
      template(send_syn([id(RepId),quote(Quote),fix(Fix),body(Body)]))
    ; nextval(syn,SynId),
      get_time(T),
      stamp_date_time(T,D,local),
      typo_dbi_assert(syn(SynId,UserId,RepId,[body(Body),fix(Fix),created_at(D),updated_at(D)])),
      redirect_to(rep,[id=RepId])
    ).
send_syn(Request) :-
    method(get,Request),
    current_user(_,_),
    http_parameters(Request,[id(RepId)],[attribute_declarations(declaration(syn))]),
    typo:rep(RepId,_,_,Rep),
    typo:quote(Rep,Quote),
    template(send_syn([id(RepId),quote(Quote),fix(Quote),body('')])).
send_syn(_) :- redirect_to(/).

send_ack(Request) :-
    method(post,Request),
    current_user(UserId,_),
    http_parameters(Request,[id(SynId),body(Body)],[attribute_declarations(declaration(ack))]),
    typo:syn(SynId,_,RepId,_),
    ( Body == ''
    ; nextval(ack,AckId),
      get_time(T),
      stamp_date_time(T,D,local),
      typo_dbi_assert(ack(AckId,UserId,SynId,[body(Body),created_at(D)]))),
    redirect_to(rep,[id=RepId]).
send_ack(_) :- redirect_to(/).

bib(isbn(13,ISBN)) --> html_post(isbn13,[dt('ISBN 13'),dd(ISBN)]).
bib(isbn(10,ISBN)) --> html_post(isbn10,[dt('ISBN 10'),dd(ISBN)]).
bib(edition(Edition)) --> html_post(edition,[dt('Edition'),dd(Edition)]).
bib(impression(Impression)) --> html_post(impression,[dt('Impression'),dd(Impression)]).
bib(create_rep(Id)) -->
    { path(create_rep,[id=Id],Path) },
    html_post(create_rep,span(['[',a(href(Path),create_rep),']'])).
bib(put_off(Id)) -->
    { path(put_off,[id=Id],Path) },
    html_post(put_off,span(['[',a(href(Path),put_off),']'])).
bib(review(Id)) -->
    { typo:review(_,Id,_,Review),
      typo:body(Review,Body),
      path(write_review,[id=Id],Path) },
    html_post(review,div([class(review)],['[',a(href(Path),'edit'),']',div(Body)])).
bib(review(Id)) -->
    { path(write_review,[id=Id],Path) },
    html_post(review,div([class(review)],['[',a(href(Path),'write review'),']'])).
bib(image(Attributes)) -->
    html_post(image,div([class(image)],[image(Attributes,[])])).
bib(Term) --> typo_post(Term).

bib(Request) :-
    http_parameters(Request,[id(BibId)],[attribute_declarations(declaration(bib))]),
    typo:bib(BibId,Bib),
    ( current_user(UserId,_),
      typo:bookshelf(_,UserId,BibId,_) ->
      template(bib([put_off(BibId),create_rep(BibId),review(BibId)|Bib]))
    ; template(bib(Bib))
    ).
bib(_) :- redirect_to(/).

rep(nickname(Name)) --> html_post(nickname,span(['reported by ',Name])).
rep(p(P)) --> html_post(p,span(['[p.',P,']'])).
rep(quote(Quote)) --> html_post(quote,[h4('Quote'),div(blockquote(pre(Quote)))]).
rep(syn(Id,UserId,RepId,Syn)) -->
    { current_user(_,CurrentUser),
      typo:nickname(CurrentUser,Nickname) },
    html_post(syn,syn(Id,UserId,RepId,Syn,[nickname(Nickname)])),!.
rep(syn(Id,UserId,RepId,Syn)) -->
    html_post(syn,syn(Id,UserId,RepId,Syn,[])).
rep(send_syn(RepId)) -->
    { current_user(_,_),
      typo_html:link(send_syn,[id=RepId],Link) },
    html_post(send_syn,div(Link)),!.
rep(send_syn(_)) --> [].
rep(Term) --> typo_post(Term).

rep(Request) :-
    http_parameters(Request,[id(RepId)],[attribute_declarations(declaration(rep))]),
    typo:rep(RepId,UserId,BibId,Rep),
    typo:bib(BibId,Bib),
    typo:title(Bib,Title),
    typo:user(UserId,User),
    typo:nickname(User,Name),
    typo:syns(RepId,Syns),
    append([title(Title),nickname(Name),send_syn(RepId)|Rep],Syns,Options),
    template(rep(Options)).
rep(_) :- redirect_to(/).

window(Request) :-
    method(get,Request),
    current_user(_,_),
    reply_typo_page(window,
                    [ \form(window,['']) ]).
window(_) :- redirect_to(/).

account(Request) :-
    method(get,Request),
    current_user(_,_),
    reply_typo_page(account,
                    []).
account(_) :- redirect_to(/).

modify_password(Id,User,Password) :-
    typo:nickname(User,Name),
    typo:mail_address(User,MailAddress),
    Option = [algorithm(sha1)],
    sha_hash(Password,Hash,Option),
    string_to_list(S,Hash),
    base64(S,Base64),
    assert(typo:user(Id,[nickname(Name),mail_address(MailAddress),password(Base64,Option)])),
    retract(typo:user(Id,User)).

modify_password(Request) :-
    current_user(Id,User),
    method(post,Request),
    http_parameters(Request,
                    [ current_password(CurrentPassword),
                      new_password(NewPassword),
                      new_password_repeat(NewPasswordRepeat)
                    ],
                    [ attribute_declarations(declaration(account))
                    ]),
    password(User,CurrentPassword),
    NewPassword = NewPasswordRepeat,
    modify_password(Id,User,NewPassword),
    reply_typo_page(modify_password,
                    [ div('Your account have been updated successfully.')
                    ]).
modify_password(_) :-
    current_user(_,_),
    reply_typo_page(modify_password,
                    [ div('If you would like to modify your account, let me know:'),
                      \form(modify_password,
                            [ \table([['current password',     input([type(password),name(current_password)])],
                                      ['new password',         input([type(password),name(new_password)])],
                                      ['new password (repeat)',input([type(password),name(new_password_repeat)])]
                                     ]),
                              div(input([type(submit)]))])
                    ]).
modify_password(_) :- redirect_to(/).

modify_mail_address(Id,User,MailAddress) :-
    typo:nickname(User,Name),
    memberchk(password(Base64,Option),User),
    assert(typo:user(Id,[nickname(Name),mail_address(MailAddress),password(Base64,Option)])),
    retract(typo:user(Id,User)).

modify_mail_address(Request) :-
    current_user(Id,User),
    method(post,Request),
    ( http_parameters(Request,
                      [ current_password(CurrentPassword),
                        mail_address(MailAddress),
                        mail_address_repeat(MailAddressRepeat)
                      ],
                      [ attribute_declarations(declaration(account))
                      ]) ->
      ( password(User,CurrentPassword),
        MailAddress = MailAddressRepeat,
        modify_mail_address(Id,User,MailAddress) ->
        reply_typo_page(modify_mail_address,
                        [ div('Your account have been updated successfully.')
                        ])
      ; template(modify_mail_address(MailAddress,MailAddressRepeat))
      )
    ; typo:mail_address(User,MailAddress),
      template(modify_mail_address(MailAddress))
    ).
modify_mail_address(Request) :-
    current_user(_,User),
    method(get,Request),
    typo:mail_address(User,MailAddress),
    template(modify_mail_address(MailAddress)).
modify_mail_address(_) :- redirect_to(/).

leave(Request) :-
    method(get,Request),
    current_user(_,User),
    nickname(User,Name),
    reply_typo_page(leave,
                    [ div(['Would you like to leave typo, i.e., to drop your account `',
                           Name,
                           '\' and withdraw some of your data?'
                          ]),
                      \form(leave,[div(\input_yes_or_no)])
                    ]).
leave(Request) :-
    method(post,Request),
    http_read_data(Request,Data,[]),
    current_user(Id,_),
    ( read_data(yes,_,Data) ->
      http_session_retractall(user(Id)),
      retractall(typo:user(Id,_)),
      redirect_to(/);
      go_to(account)
    ).
leave(_) :- redirect_to(/).

%% templates by reply_html_page
template(join(Name)) :-
    page_title(join,Title),
    reply_html_page([ link([type('text/css'),rel('stylesheet'),href('/stylesheets/typo.css')]),
                      Title
                    ],
                    [ \logo,
                      div('Let me know:'),
                      \form(join,
                            [ \table([['nickname',             input([type(text),name(nickname),value(Name)])],
                                      ['password',             input([type(password),name(password)])],
                                      ['password (repeat)',    input([type(password),name(password_repeat)])],
                                      ['mail address',         input([type(text),name(mail_address)])],
                                      ['mail address (repeat)',input([type(text),name(mail_address_repeat)])]]),
                              div(input([type(submit)]))]),
                      \copyright
                    ]).

template(create_rep(BibId,[caption(Caption),p(P),quote(Quote),fix(Fix)])) :-
    typo:bib(BibId,Bib),
    typo:title(Bib,Title),
    reply_typo_page(create_rep,
                    [ div(['You are going to create a new rep for the bib `',Title,'\'.']),
                      div('Let me know:'),
                      \form(create_rep,
                            [ \table([['Caption',input([type(text),name(caption),value(Caption)])],
                                      ['Page',input([type(text),name(p),value(P)])],
                                      ['Quote',textarea(name(quote),Quote)],
                                      ['Fix',textarea(name(fix),Fix)]
                                     ]),
                              input([type(hidden),name(id),value(BibId)]),
                              input([type(submit),value('create')])
                            ])
                    ]).

template(put_on) :-
    template(put_on([message('Let me know:'),title(''),isbn(''),edition(''),impression('')])).
template(put_on(Options)) :-
    maplist_bs(put_on,Options,Posts),
    reply_typo_page(put_on,
                    [ div(\html_receive(message)),
                      \form(put_on,
                            [ \html_receive(image),
                              \table([['Title',\html_receive(title)],
                                      ['ISBN ',\html_receive(isbn)],
                                      ['Edition',\html_receive(edition)],
                                      ['Impression',\html_receive(impression)]
                                     ]),
                              \html_receive(force),
                              input([type(submit),value('put on')])
                            ])
                    | Posts]).

template(modify_mail_address(MailAddress)) :-
    template(modify_mail_address(MailAddress,MailAddress)).
template(modify_mail_address(MailAddress,MailAddressRepeat)) :-
    reply_typo_page(modify_mail_address,
                    [ div('If you would like to modify your account, let me know:'),
                      \form(modify_mail_address,
                            [ \table([['password',input([type(password),name(current_password)])],
                                      ['mail address',input([type(text),name(mail_address),value(MailAddress)])],
                                      ['mail address (repeat)',input([type(text),name(mail_address_repeat),value(MailAddressRepeat)])]]),
                              div(input([type(submit)]))])
                    ]).

template(bib(Options)) :-
    maplist_bs(bib,Options,Posts),
    reply_typo_page(bib,
                    [ h2(\html_receive(title)),
                      div([\html_receive(create_rep),\html_receive(put_off)]),
                      \html_receive(image),
                      div([class(bib)],
                          [dl([\html_receive(isbn13),
                               \html_receive(isbn10),
                               \html_receive(edition),
                               \html_receive(impression)
                             ])]),
                      \html_receive(review)
                    | Posts]).

template(rep(Options)) :-
    maplist_bs(rep,Options,Posts),
    reply_typo_page(rep,
                    [ h2(\html_receive(title)),
                      h3([\html_receive(p),
                           \html_receive(caption),
                           ' / ',
                           \html_receive(nickname)
                          ]),
                      \html_receive(quote),
                      \html_receive(send_syn),
                      \html_receive(syn,syn_html)
                    | Posts]).

template(send_syn(Options)) :-
    option(id(Id),Options),
    option(quote(Quote),Options),
    option(fix(Fix),Options),
    option(body(Body),Options),
    reply_typo_page(send_syn,
                    [ div('Let me know:'),
                      \form(send_syn,
                            [ input([type(hidden),name(id),value(Id)]),
                              \table([['quote',Quote],
                                      ['alternative fix',textarea([name(fix)],Fix)],
                                      ['message',textarea([name(body)],Body)]]),
                              div(input([type(submit)]))
                            ])
                    ]).
