;;
;; Test aws
;;

(use gauche.test)

(test-start "aws")
(use aws)
(test-module 'aws)

(test-section "parameters")

(use gauche.parameter)

(test* "aws-access-key-id" #t (is-a? aws-access-key-id <parameter>))

;; CAUTION: the access key id must be secret.
(aws-access-key-id "********************")

(test* "aws-locale" #t (is-a? aws-locale <parameter>))
(test* "aws-locale" 'us (aws-locale))
(aws-locale 'jp)

(test* "aws-endpoint-host-name" "ecs.amazonaws.jp" (aws-endpoint-host-name))
(test* "aws-xslt-endpoint-host-name" "xml-jp.amznxslt.com" (aws-xslt-endpoint-host-name))


(test-section "batch requests")


(test-section "multiple operation requests")
;; examples:
;; -
;; http://webservices.amazon.com/onca/xml?
;; Service=AWSECommerceService&
;; AWSAccessKeyId=[Access Key ID]&
;; Operation=ItemLookup,SellerLookup&
;; ItemLookup.1.ItemId=0976925524&
;; SellerLookup.1.SellerId=[ID]
;; -
;; http://webservices.amazon.com/onca/xml?
;; Service=AWSECommerceService&
;; ASWAccessKeyId=[Access Key ID]&
;; Operation=ItemLookup,SellerLookup&
;; ItemLookup.1.ItemId=0976925524&
;; ItemLookup.2.ItemId=123456789&
;; ItemLookup.Shared.IdType=ASIN&
;; SellerLookup.1.SellerId=[ID]


(test-section "errors")
;; examples:
;; <Errors>
;;   <Error>
;;     <Code>missing_service_parameter</code>
;;     <message>Your request is missing the Service parameter. Please add the Service parameter to your request and retry. Valid values for the Service parameter include AWSEcommerceService.</message>
;;   </Error>
;; </Errors>


(test-section "finding items to buy")
;; examples:
;; <Item>
;;   ...
;;   <Offer>
;;     ...
;;     <OfferListing>
;;       <OfferListingId>g7CWSnBZmVESKy%2BCNS</OfferListingId>
;;       <ExchangeId>Y01Y4038850Y8583882</ExchangeId>
;;       <Price>
;;         <Amount>625</Amount>
;;         <CurrencyCode>USD</CurrencyCode>
;;         <FormattedPrice>$6.25</FormattedPrice>
;;       </Price>
;;       <Availability>Usually ships in 1-2 business days 
;;       </Availability>
;;     </OfferListing>
;;   </Offer>
;; </Item>


(test-section "operations")

(let ((query (aws-help :about 'help)))
  (test* "aws-help (query->uri)"
         "http://ecs.amazonaws.jp/onca/xml?AWSAccessKeyId=********************&Service=AWSECommerceService&Operation=Help&About=help"
         (query->uri query))
  (test* "aws-help (execute)" "200" (execute query))
  (print (ref query 'body))
  )
(let ((query (aws-browse-node-lookup :browse-node-id 1)))
  (test* "aws-browse-node-lookup (query->uri)"
         "http://ecs.amazonaws.jp/onca/xml?AWSAccessKeyId=********************&Service=AWSECommerceService&Operation=BrowseNodeLookup&BrowseNodeId=1"
         (query->uri query))
  (test* "aws-browse-node-lookup (execute)" "200" (execute query))
  (print (ref query 'body))
  )
(let ((query (aws-cart-add :cart-id 0 :hmac 0 :quantity 100)))
  (test* "aws-cart-add (query->uri)"
         "http://ecs.amazonaws.jp/onca/xml?AWSAccessKeyId=********************&Service=AWSECommerceService&Operation=CartAdd&CartId=0&HMAC=0&Quantity=100"
         (query->uri query))
  (test* "aws-cart-add" "200" (execute query))
  (print (ref query 'body))
  )
(let ((query (aws-item-lookup :item-id "9784756134141" :id-type "ISBN" :search-index "Books")))
  (test* "aws-item-lookup (query-uri)"
         "http://ecs.amazonaws.jp/onca/xml?AWSAccessKeyId=********************&Service=AWSECommerceService&Operation=ItemLookup&ItemId=9784756134141&IdType=ISBN&SearchIndex=Books"
         (query->uri query))
  (test* "aws-item-lookup" "200" (execute query))
  (print (ref query 'body))
  )
(for-each
 (lambda (isbn)
   (let ((query (aws-item-lookup :item-id isbn :id-type "ISBN" :search-index "Books" :response-group "Medium")))
     (test* "aws-item-lookup" "200" (execute query))
     (print (ref query 'body))
     ))
 '("9784756134141"
   "4756136494"
   "9784873113357"))
(let ((query (aws-item-search :title "Pthreads" :search-index "Books")))
  (test* "aws-item-search" "200" (execute query))
  (print (ref query 'body))
  )
(let ((query (aws-item-search :title "Pthreads" :search-index "Books" :response-group "Large")))
  (test* "aws-item-search" "200" (execute query))
  (print (ref query 'body))
  )

(test-end)
