;; -*- mode: scheme; coding: utf-8 -*-
;;
;; Test typo.sql
;;

(use gauche.test)

(test-start "typo.sql")
(use typo.sql)
(test-module 'typo.sql)

(test-section "typo-underscore")
(for-each
 (lambda (x expected)
   (test* "typo-underscore" expected (typo-underscore x)))
 '("SlimTower" "PDF-Draft" "あいう-えお")
 '("slimtower" "pdf_draft" "あいう_えお"))

(define-syntax test*-with-pairs
  (syntax-rules ()
    ((_ proc)
     '())
    ((_ proc (x y ...) data ...)
     (begin
       (call-with-values
           (lambda () (apply proc '(y ...)))
         (lambda (statement fields)
           (test* (x->string 'proc) 'x statement)))
       (test*-with-pairs proc data ...)))))

(test-section "typo-sql-where")
(test*-with-pairs
 typo-sql-where
 ("")
 ("WHERE id = ?" 1)
 ("WHERE id = ? AND abc = ?" :id 1 :abc xyz))

(test-section "typo-sql-select")
(test*-with-pairs
 typo-sql-select
 ("SELECT id, name, address FROM foos " "foos" (id name address))
 ("SELECT name FROM bars WHERE id = ?" "bars" (name) 100)
 ("SELECT * FROM bazs WHERE name = ? AND fee = ?" "bazs" () :name "Takeshi Abe" :fee "$3,000")
 ("SELECT * FROM bazs WHERE name = ? AND fee IS NULL" "bazs" () :name "Alice" :fee #f)
)

(test-section "typo-sql-count")
(test*-with-pairs
 typo-sql-count
 ("SELECT count(*) FROM users " "users")
 ("SELECT count(*) FROM bibs WHERE caption = ? AND p IS NULL" "bibs" :caption "XYZ" :p #f)
)

(test-section "typo-sql-insert")
(test*-with-pairs
 typo-sql-insert
 ("INSERT INTO members (name) VALUES (?)" "members" :name "Foo Bar")
 ("INSERT INTO members (aa, bb, cc) VALUES (?, ?, ?)" "members" :aa "" :bb #f :cc 100))

(test-section "typo-sql-insert-from-port")
(test*-with-pairs
 (lambda (table str) (call-with-input-string str (cut typo-sql-insert-from-port table <>)))
 (("INSERT INTO members (name, birthdate) VALUES (?, ?)"
   "INSERT INTO members (name, birthdate) VALUES (?, ?)")
  "members"
  "(foo :name \"Bar Baz\" :birthdate (2007 10 23))\n(alice :name \"Alice Hacker\" :birthdate (1981 5 7))")
 (("INSERT INTO members (name) VALUES (?)"
   "INSERT INTO members (name) VALUES (?)")
  "members"
   "(japan :name 'ja) (:name en)"))
(for-each
 (lambda (bad)
   (test* "typo-sql-insert-from-port"
          *test-error*
          (call-with-input-string bad (cut typo-sql-insert-from-port "members" <>))))
 '("()" "(foo)" ")"))

(test-section "typo-sql-update")
(test*-with-pairs
 typo-sql-update
 ("UPDATE octopi SET name = ?, weight = ? WHERE id = ?" "octopi" (:name "Mary" :weight "700") 7)
 ("UPDATE octopi SET price = ? WHERE weight = ? AND color = ?" "octopi" (:price 100) :weight 500 :color 'red)
 ("UPDATE lives SET status = ? WHERE status IS NULL AND error = ?" "lives" (:status 0) :status #f :error 5)
 ("UPDATE lives SET error = ?, status = NULL WHERE status = ?" "lives" (:error 20 :status #f) :status 0)
 )

(test-section "typo-sql-delete")
(test*-with-pairs
 typo-sql-delete
 ("DELETE FROM cars WHERE id = ?" cars 32)
 ("DELETE FROM mobile_phones WHERE name = ? AND version = ?" mobile-phones :name "XXX" :version 2.0)
 ("DELETE FROM books WHERE title = ? AND isbn = ?" books :title "Emacs Lisp Reference Manual" :isbn "*************"))

(test-section "typo-sql-nextval")
(test*-with-pairs
 typo-sql-nextval
 ("SELECT nextval(?)" "member_id_seq"))

(test-section "typo-sql-create-table")
(test*-with-pairs
 typo-sql-create-table
 ("CREATE TABLE trees (id SERIAL PRIMARY KEY, name varchar NOT NULL, age integer)" trees :name (<string> :null #f) :age (<integer>))
 ("CREATE TABLE manuals (id SERIAL PRIMARY KEY, name varchar(32) NOT NULL, description varchar, author varchar(32) NOT NULL DEFAULT ?)" "manuals" :name ((<string> 32) :null #f) :description (<string>) :author ((<string> 32) :null #f :default "Takeshi Abe"))
 ("CREATE TABLE no_such_tables (id SERIAL PRIMARY KEY, name varchar(256) NOT NULL UNIQUE, foo_id integer NOT NULL REFERENCES foo(id))" no-such-tables :name ((<string> 256) :null #f :unique #t) :foo (:foreign :null #f))
 ("CREATE TABLE polies (id SERIAL PRIMARY KEY, source_id integer NOT NULL, source_type varchar NOT NULL, target_id integer NOT NULL, target_type varchar NOT NULL)" polies :source (:polymorphic :null #f) :target (:polymorphic :null #f))
 ("CREATE TABLE flowers (id SERIAL PRIMARY KEY, name varchar(1024), flavor varchar(32), UNIQUE (name, flavor))" flowers :name ((<string> 1024)) :flavor ((<string> 32)) :unique (name flavor))
 )

(test-section "typo-sql-drop-table")
(test*-with-pairs
 typo-sql-drop-table
 ("DROP TABLE keyboards" keyboards))

(test-section "typo-sql-drop-sequence")
(test*-with-pairs
 typo-sql-drop-sequence
 ("DROP SEQUENCE a_b_seq" a-b-seq))

(test-section "typo-sql-drop-view")
(test*-with-pairs
 typo-sql-drop-view
 ("DROP VIEW relaxins" relaxins))

(test-end)
