;;
;; Test text.inflection
;;

(use gauche.test)

(test-start "text.inflection")
(use text.inflection)
(test-module 'text.inflection)

(use srfi-19)
(use text.tree)
(use util.list)

(define *singular-plural*
 '((apple . apples)
   (beattle . beattles)
   (comic . comics)

   (equipment . equipment)
   (information . information)
   (rice . rice)
   (money . money)
   (spices . spices)
   (series . series)
   (fish . fish)
   (sheep . sheep)

   (person . people)
   (man . men)
   (child . children)
   (sex . sexes)
   (move . moves)

   (quiz . quizzes)
   (ox . oxen)
   (mouse . mice)
   (louse . lice)
   (matrix . matrices)
   (vertex . vertices)
   (dish . dishes)
   (handy . handies)
   (hive . hives)
   (knife . knives)
   (shelf . shelves)
   (buffalo . buffaloes)
   (tomato . tomatoes)
   (bus . buses)
   (alias . aliases)
   (octopus . octopi)
   (axis . axes)
   )
 )

(define *pluralize-only*
  '((basis . bases)))

(define *singularize-only*
  '((base . bases)))

(test-section "pluralize")
(for-each
 (lambda (pair)
   (let ((s (x->string (car pair)))
         (p (x->string (cdr pair))))
     (test* "test-pluralize [singular -> plural]" p (pluralize s))
;;     (test* "pluralize [plural -> plural]"   p (pluralize p))
     ))
 (append *singular-plural* *pluralize-only*))

(test-section "singularize")
(for-each
 (lambda (pair)
   (let ((s (x->string (car pair)))
         (p (x->string (cdr pair))))
;;     (test* "singularize [singular -> singular]" s (singularize s))
     (test* "singularize [plural -> singular]"   s (singularize p))
     ))
 (append *singular-plural* *singularize-only*))

(define *ordinal*
  '((0 "0th" "zeroth")
    (1 "1st" "first")
    (2 "2nd" "second")
    (3 "3rd" "third")
    (4 "4th" "fourth")
    (5 "5th" "fifth")
    (6 "6th" "sixth")
    (7 "7th" "seventh")
    (8 "8th" "eighth")
    (9 "9th" "ninth")
    (10 "10th" "tenth")
    (11 "11th" "eleventh")
    (12 "12th" "twelfth")
    ))

(test-section "ordinalize")
(for-each
 (lambda (pair)
   (test* "ordinalize" (x->string (cadr pair)) (ordinalize (car pair))))
 *ordinal*)

(test-section "dasherize")
(for-each
 (lambda (pair)
   (test* "dasherize(camelcase)" (x->string (cdr pair)) (dasherize (car pair))))
 '((PDFReferenceManual . pdf-reference-manual)
   (WiLiKi . wi-li-ki)
   (W-ZERO3 . w-zero3)
   ))
(for-each
 (lambda (pair)
   (test* "dasherize(underscore)" (x->string (cdr pair)) (dasherize (car pair) :from 'underscore)))
 '((ABC_DEFG . abc-defg)
   (__3 . "--3")
   (1__ . "1--")
   ))

(test-section "camelize")
(for-each
 (lambda (pair)
   (test* "camelize(dash)" (x->string (cdr pair)) (camelize (car pair))))
 '((a-bc-d . ABcD)
   (-qwerty . Qwerty)
   (b0c- . B0C)
   (xy-99 . Xy99)
   ))
(for-each
 (lambda (pair)
   (test* "camelize(underscore)" (x->string (cdr pair)) (camelize (car pair) :from 'underscore)))
 '((a_bc_d . ABcD)
   (_qwerty . Qwerty)
   (b0c_ . B0C)
   (xy_000 . Xy000)
   ))
(let ((table (alist->hash-table '(("bc" . "BBBBCCCC") ("xy" . "XY")) 'string=?)))
  (for-each
   (lambda (pair)
     (test* "camelize(table)" (x->string (cdr pair)) (camelize (car pair) :table table)))
   '((a-bc-d . ABBBBCCCCD)
     (-qwerty . Qwerty)
     (b0c- . B0C)
     (xy-000 . XY000))))

(test-end)
